#include <stdlib.h>
#include "anamod.h"
#include "anamodsalsamodules.h"
#include "petscmat.h"

/*
 * test the simple category
 */
int main(int argc,char **argv)
{
  Mat A; 
  PetscScalar v,vd=3.; PetscInt i,ip;
  AnalysisItem result;
  PetscBool flg; PetscErrorCode ierr;
  PetscInitialize(&argc,&argv,0,0);
  ierr = AnaModInitialize(); CHKERRQ(ierr);

  ierr = RegisterSimpleModules(); CHKERRQ(ierr);

  ierr = MatCreateSeqAIJ(MPI_COMM_SELF,4,4,0,0,&A); CHKERRQ(ierr);
  for (i=0; i<4; i++) {
    v=vd;
    ierr = MatSetValues(A,1,&i,1,&i,&v,INSERT_VALUES); CHKERRQ(ierr);
  }
  for (i=0; i<3; i++) {
    ip=i+1;
    PetscScalar n=1.,m=-1.;
    ierr = MatSetValues(A,1,&i,1,&ip,&m,INSERT_VALUES); CHKERRQ(ierr);
    ierr = MatSetValues(A,1,&ip,1,&i,&n,INSERT_VALUES); CHKERRQ(ierr);
  }
  ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
  ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"simple","diagonal-dominance",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (result.r!=vd-2.)
    SETERRQ1(1,"incorrect ddom1: %e",result.r);

  i = 3; v = -vd;
  ierr = MatSetValues(A,1,&i,1,&i,&v,INSERT_VALUES); CHKERRQ(ierr);
  ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
  ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"simple","trace",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (result.r!=2*vd)
    SETERRQ1(1,"incorrect trace: %e",result.r);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"simple","trace-abs",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (result.r!=4*vd)
    SETERRQ1(1,"incorrect trace-abs: %e",result.r);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"simple","symmetry-snorm",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (result.r!=vd)
    SETERRQ1(1,"incorrect snorm: %e",result.r);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"simple","symmetry-anorm",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (result.r!=2.)
    SETERRQ1(1,"incorrect anorm: %e",result.r);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"simple","diagonal-dominance",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (result.r!=-vd-1.)
    SETERRQ1(1,"incorrect ddom2: %e",result.r);

  i = 0; ip = 2; v = 2.;
  ierr = MatSetValues(A,1,&i,1,&ip,&v,INSERT_VALUES); CHKERRQ(ierr);
  i = 0; ip = 3; v = 2.;
  ierr = MatSetValues(A,1,&i,1,&ip,&v,INSERT_VALUES); CHKERRQ(ierr);
  ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
  ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"simple","norm1",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (result.r!=vd+4.)
    SETERRQ1(1,"incorrect norm1: %e",result.r);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"simple","normInf",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (result.r!=vd+5.)
    SETERRQ1(1,"incorrect normInf: %e",result.r);


  ierr = MatDestroy(A); CHKERRQ(ierr);
  ierr = AnaModFinalize(); CHKERRQ(ierr);
  PetscFinalize();
  PetscFunctionReturn(0);
}
