#include <stdlib.h>
#include "syspro.h"

/* test for problem solving without preprocessors:
   the 'problem' is to copy a number
*/

static PetscErrorCode solvebycopy
(NumericalProblem problem,void *dum,NumericalSolution *rsol)
{
  NumericalSolution sol; PetscErrorCode ierr;
  PetscFunctionBegin;
  ierr = PetscMalloc(sizeof(int),(int**)&sol); CHKERRQ(ierr);
  *(int*)sol = *(int*)problem;
  *rsol = sol;
  PetscFunctionReturn(0);
}

int main(int argc,char **argv) {
  NumericalProblem problem; NumericalSolution solution;
  PetscErrorCode ierr;
  PetscInitialize(&argc,&argv,0,0);
  ierr = SysProInitialize(); CHKERRQ(ierr);
  ierr = SysProDeclareFunctions
    (NULL,NULL,NULL,solvebycopy,NULL,NULL,NULL,NULL,NULL,NULL,NULL); CHKERRQ(ierr);
  
  /* set up a simple problem */
  ierr = PetscMalloc(sizeof(int),(int**)&problem); CHKERRQ(ierr);
  *(int*)problem = 5;

  /* solve this problem */
  ierr = PreprocessedProblemSolving(problem,&solution); CHKERRQ(ierr);

  /* check for correctness */
  {
    int s = *(int*)solution;
    if (s!=5)
      SETERRQ1(PETSC_COMM_SELF,1,"Computed wrong solution %d",s);
  }

  ierr = PetscFree(problem); CHKERRQ(ierr);
  ierr = PetscFree(solution); CHKERRQ(ierr);
  ierr = SysProFinalize(); CHKERRQ(ierr);
  PetscFinalize();
  return 0;
}
