#include <stdlib.h>
#include "syspro.h"
#include "sysprotransform.h"
#include "string.h"

/*
  Test for a single preprocessor, which adds 1 to the problem number
 */

/* the problem solving function: just a simple copy */
#undef __FUNCT__
#define __FUNCT__ "solvebycopy"
static PetscErrorCode solvebycopy
(NumericalProblem problem,void *dum,NumericalSolution *rsol)
{
  NumericalSolution sol; PetscErrorCode ierr;
  PetscFunctionBegin;
  ierr = PetscMalloc(sizeof(int),(int**)&sol); CHKERRQ(ierr);
  SysProTraceMessage("Solving problem %d\n",*(int*)problem);
  *(int*)sol = *(int*)problem;
  *rsol = sol;
  PetscFunctionReturn(0);
}

/* deallocate a problem object */
#undef __FUNCT__
#define __FUNCT__ "delprob"
static PetscErrorCode delprob(NumericalProblem p)
{
  PetscErrorCode ierr;
  PetscFunctionBegin;
  ierr = PetscFree(p); CHKERRQ(ierr);
  PetscFunctionReturn(0);
}

/* allocate a solution object */
#undef __FUNCT__
#define __FUNCT__ "makesol"
static PetscErrorCode makesol(NumericalProblem p,NumericalSolution *rs)
{
  NumericalSolution s; PetscErrorCode ierr;
  PetscFunctionBegin;
  ierr = PetscMalloc(sizeof(int),(int**)&s); CHKERRQ(ierr);
  *rs = s;
  PetscFunctionReturn(0);
}

/* deallocate a solution object */
#undef __FUNCT__
#define __FUNCT__ "delsol"
static PetscErrorCode delsol(NumericalSolution s)
{
  PetscErrorCode ierr;
  PetscFunctionBegin;
  ierr = PetscFree(s); CHKERRQ(ierr);
  PetscFunctionReturn(0);
}

/* preprocessor handling: we add 1 to the problem value */
#undef __FUNCT__
#define __FUNCT__ "adder"
static PetscErrorCode adder
(const char *choice,int optionvalue,PetscBool overwrite,
 NumericalProblem oldproblem,NumericalProblem *rnew,
 void *ctx,void **lctx,PetscBool *success)
{
  NumericalProblem newproblem; PetscErrorCode ierr;
  PetscFunctionBegin;
  if (strcmp(choice,"add1")) SETERRQ1(PETSC_COMM_SELF,1,"Unknown choice <%s>",choice);

  ierr = PetscMalloc(sizeof(int),(int**)&newproblem); CHKERRQ(ierr);
  *(int*)newproblem = *(int*)oldproblem+1;
  SysProTraceMessage("new problem: %d\n",*(int*)newproblem);
  *rnew = newproblem;

  *success = PETSC_TRUE;
  PetscFunctionReturn(0);
}

/* the unpreprocess function subtracts one */
#undef __FUNCT__
#define __FUNCT__ "unadder"
static PetscErrorCode unadder
(const char *choice,PetscBool overwrite,void *ctx,void *lctx,
 NumericalProblem pproblem,NumericalProblem oproblem,
 NumericalSolution psol,NumericalSolution osol)
{
  PetscFunctionBegin;
  if (strcmp(choice,"add1")) SETERRQ1(PETSC_COMM_SELF,1,"Unknown choice <%s>",choice);
  *(int*)osol = *(int*)psol-1;
  PetscFunctionReturn(0);
}

/* we declare our one and only preprocessor */
#undef __FUNCT__
#define __FUNCT__ "declareadders"
static PetscErrorCode declareadders()
{
  PetscErrorCode ierr;
  PetscFunctionBegin;
  ierr = NewTransformObject("adder","add1",NULL); CHKERRQ(ierr);
  PetscFunctionReturn(0);
}

#undef __FUNCT__
#define __FUNCT__ "main"
int main(int argc,char **argv) {
  NumericalProblem problem; NumericalSolution solution;
  PetscErrorCode ierr;
  PetscInitialize(&argc,&argv,0,0);
  ierr = SysProInitialize(); CHKERRQ(ierr);
  ierr = SysProDeclareFunctions
    (NULL,NULL,NULL,solvebycopy,delprob,
     makesol,NULL,delsol,NULL,NULL,NULL); CHKERRQ(ierr);
  ierr = SysProDeclareTraceFunction(&SysProDefaultTrace); CHKERRQ(ierr);

  /* set up a simple problem */
  ierr = PetscMalloc(sizeof(int),(int**)&problem); CHKERRQ(ierr);
  *(int*)problem = 5;

  /* declare a preprocessor */
  ierr = DeclarePreprocessor
    ("adder",&declareadders,NULL,NULL,NULL,NULL,NULL,
     &adder,&unadder); CHKERRQ(ierr);

  /* solve this problem */
  ierr = PreprocessedProblemSolving(problem,&solution); CHKERRQ(ierr);

  /* check for correctness */
  {
    int s = *(int*)solution;
    if (s!=5)
      SETERRQ1(PETSC_COMM_SELF,1,"Computed wrong solution %d",s);
  }

  ierr = PetscFree(problem); CHKERRQ(ierr);
  ierr = PetscFree(solution); CHKERRQ(ierr);
  ierr = SysProFinalize(); CHKERRQ(ierr);
  PetscFinalize();
  return 0;
}
